/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.h
  * @brief          : Header for main.c file.
  *                   This file contains the common defines of the application.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MAIN_H
#define __MAIN_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "stm32f1xx_hal.h"
#include "stm32f1xx_ll_rtc.h"
#include "stm32f1xx_ll_bus.h"
#include "stm32f1xx_ll_cortex.h"
#include "stm32f1xx_ll_rcc.h"
#include "stm32f1xx_ll_system.h"
#include "stm32f1xx_ll_utils.h"
#include "stm32f1xx_ll_pwr.h"
#include "stm32f1xx_ll_gpio.h"
#include "stm32f1xx_ll_dma.h"

#include "stm32f1xx_ll_exti.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "rtc_stm32f1.h"

/* USER CODE END Includes */

/* Exported types ------------------------------------------------------------*/
/* USER CODE BEGIN ET */


typedef struct
{
	char szFilename[12];
}structLogtoFile;

#define DATASIZE 300

typedef struct
{
	const int Size;
	int iNext;
	float fArray[DATASIZE];
	
} structData;

typedef struct
{
	unsigned long ulTick;
	unsigned long ulToggle;
	unsigned long ulPeriode;
	float fValue;	
} structPWMOut;

typedef struct
{
	unsigned long ulPeriode;
	unsigned long ulTick;
	float w, en, en_1, un, un_1;
	float I_1;
	float D_1;
	float P, I, D;
} structPID;

typedef struct
{
	float kr;
	float tN, tV, t1, T;
	float tPWM;
} structPIDSettings;

typedef enum {ModeActive, ModeLog} enumModetype;

#define ADR_SETTINGS 0
typedef enum {Settingschar, Settingsint, Settingslong, Settingsulong, Settingsfloat, Settingsdouble, Settingssz} enumSettingtype;
typedef struct
{
	enumSettingtype Settingstype;
	const char * const szDescription;
	void * pSetting;
} structSettingsIndex;

typedef struct
{
	//SMS
	char SMSNumber[20];
	//Weight
	char ADCSpeedSetting;
	long lOffset;
	double dFactor;
	float fCalWeight;
	float fCalTemp;
	float fdwdt;
	float fWeightOffset;
	//Touch
	unsigned long ulSET_REG_CTOUCH_TRANSFORM_A;
	unsigned long ulSET_REG_CTOUCH_TRANSFORM_B;
	unsigned long ulSET_REG_CTOUCH_TRANSFORM_C;
	unsigned long ulSET_REG_CTOUCH_TRANSFORM_D;
	unsigned long ulSET_REG_CTOUCH_TRANSFORM_E;
	unsigned long ulSET_REG_CTOUCH_TRANSFORM_F;
	//PID
//	structPIDSettings PID;
//	float fTempSet;
//	float fueOverride;

	//ThingSpeak
	char LogToThingSpeak;
	unsigned int LogToThingSpeakInterval;
	char ThingSpeakKey[20];
	
	//SD
	char LogToSD;

	//Email
	char SMTPServer[20];
	char SMTPUser[20];
	char SMTPPass[20];
	char EmailFrom[40];
	char FromName[20];
	char EmailTo[40];
	char ToName[20];
	
	//Diagram
	float DiagramMinY;
	float DiagramMaxY;
	
	//SMS Alarm
	char cSMSAlarm;
	float fAlarmDiff;
	
	float fWeightBevor;
		
} structSettings;


int GetSizeofSettingsIndex(void);

typedef struct
{
	char PWRSIM800;
	char PWRFT800;
	char PWRAnalog;
	char PWRSD;
} structPWRStatus;

extern structPID PID;

extern long datasum; //Raw ADC Data
extern float fweight1;
extern float fweightcalibrated;
extern float ftemp1, ftemp2, ftemp3;
extern float DHTTemp, DHTHumidity;

extern float fBatVoltage;

extern structSettings Settings;

/* USER CODE END ET */

/* Exported constants --------------------------------------------------------*/
/* USER CODE BEGIN EC */
extern SPI_HandleTypeDef hspi2;
//extern RTCTIME rtcTime;
extern structData Data;
extern UART_HandleTypeDef huart1;
extern UART_HandleTypeDef huart2;
extern char bSIM800Buffer;
extern char *pSIM800Buffer;
extern char *pSIM800LastCommand;
extern char SIM800Buffer[100];
extern RTCTIME rtcTime;

/* USER CODE END EC */

/* Exported macro ------------------------------------------------------------*/
/* USER CODE BEGIN EM */

/* USER CODE END EM */

/* Exported functions prototypes ---------------------------------------------*/
void Error_Handler(void);

/* USER CODE BEGIN EFP */

int CriticalTasks(void);
int NonCriticalTasks(void);
int DebugMonitorTasks(void);

void LED_Red_ON(void);
void LED_Green_ON(void);
void LED_Red_OFF(void);
void LED_Green_OFF(void);
void PWR_3_3VD_ON(void);
void PWR_3_3VD_OFF(void);
void PWR_AVBAT_ON(void);
void PWR_AVBAT_OFF(void);
void PWR_4V_ON(void);
void PWR_4V_OFF(void);
void PWR_FT800_ON(void);
void PWR_FT800_OFF(void);
void PWR_SD_ON(void);
void PWR_SD_OFF(void);
void PWR_BACKLIGHT_ON(void);
void PWR_BACKLIGHT_OFF(void);

void FT800_SPI_ENABLE(void);
void FT800_SPI_DISABLE(void);

void EE_CS_ENABLE(void);
void EE_CS_DISABLE(void);

void SD_SPI_ENABLE(void);
void SD_SPI_DISABLE(void);

void SD_CS_ENABLE(void);
void SD_CS_DISABLE(void);

void SIM800_PWRKEY_LOW(void);
void SIM800_PWRKEY_HIGH(void);

void SIM800_Port_OFF(void);
void SIM800_Port_ON(void);

void FT800_Port_OFF(void);
void FT800_Port_ON(void);

void SD_Port_OFF(void);
void SD_Port_ON(void);
void Init_SD(void);
void Init_SD2(void);

void Analog_Port_OFF(void);
void Analog_Port_ON(void);

//ISR Callbacks
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin);

//DATA
int InitData(structData *pData);
int AddData(structData *pData, float NewValue);

//Settings
int 	InitSettings(structSettings *Settings);
int SaveSettings(structSettings *pSettings);
int PrintSettings(structSettings *pSettings);

//PWMOut
int	InitPWMOut(structPWMOut *pPWMOut);
int SetPWMOutValue(structPWMOut *pPWMOut, float fValue);

//PIDOut
int	InitPID(structPID *pPID);
int	PIDSeten(structPID *pPID, float fValue);
int PIDSetw(structPID *pPID, float fValue);
int CalculatePID(structPID *pPID);

//LogtoFile
int InitLogtoFile(structLogtoFile *p);
int LogtoFile(structLogtoFile *p);

// SIM800Buffer
int SIM800Buffer_Flush(void);

void SDDebug(void);
int xatoi (			/* 0:Failed, 1:Successful */
	char **str,		/* Pointer to pointer to the string */
	long *res		/* Pointer to the valiable to store the value */
);

void put_dump (
	const void* buff,		/* Pointer to the array to be dumped */
	unsigned long addr,		/* Heading address value */
	int len,				/* Number of items to be dumped */
	int width				/* Size of the items (DW_CHAR, DW_SHORT, DW_LONG) */
);

int xgets (		/* 0:End of stream, 1:A line arrived */
	char* buff,	/* Pointer to the buffer */
	int len		/* Buffer length */
);

void delayus(int useconds);

void _Error_Handler(char * file, int line);

/* USER CODE END EFP */

/* Private defines -----------------------------------------------------------*/
#define FT800_SPI_nEN_Pin GPIO_PIN_0
#define FT800_SPI_nEN_GPIO_Port GPIOC
#define EN_4V_Pin GPIO_PIN_1
#define EN_4V_GPIO_Port GPIOC
#define SIM800_PWRKEY_Pin GPIO_PIN_2
#define SIM800_PWRKEY_GPIO_Port GPIOC
#define SIM800_RING_Pin GPIO_PIN_3
#define SIM800_RING_GPIO_Port GPIOC
#define SYS_WKUP_Pin GPIO_PIN_0
#define SYS_WKUP_GPIO_Port GPIOA
#define SYS_WKUP_EXTI_IRQn EXTI0_IRQn
#define SIM800_DTR_Pin GPIO_PIN_1
#define SIM800_DTR_GPIO_Port GPIOA
#define SIM800_TX_Pin GPIO_PIN_2
#define SIM800_TX_GPIO_Port GPIOA
#define SIM800_RX_Pin GPIO_PIN_3
#define SIM800_RX_GPIO_Port GPIOA
#define AD7794_nCS_Pin GPIO_PIN_4
#define AD7794_nCS_GPIO_Port GPIOA
#define ADC_VBAT_Pin GPIO_PIN_4
#define ADC_VBAT_GPIO_Port GPIOC
#define ADC_RES_Pin GPIO_PIN_5
#define ADC_RES_GPIO_Port GPIOC
#define PB0_Pin GPIO_PIN_0
#define PB0_GPIO_Port GPIOB
#define PB1_Pin GPIO_PIN_1
#define PB1_GPIO_Port GPIOB
#define EN_AVBAT_Pin GPIO_PIN_2
#define EN_AVBAT_GPIO_Port GPIOB
#define LED_GREEN_Pin GPIO_PIN_10
#define LED_GREEN_GPIO_Port GPIOB
#define LED_RED_Pin GPIO_PIN_11
#define LED_RED_GPIO_Port GPIOB
#define EEPROM_nCS_Pin GPIO_PIN_12
#define EEPROM_nCS_GPIO_Port GPIOB
#define DS_1W_1_Pin GPIO_PIN_6
#define DS_1W_1_GPIO_Port GPIOC
#define DS_1W_2_Pin GPIO_PIN_7
#define DS_1W_2_GPIO_Port GPIOC
#define DS_1W_3_Pin GPIO_PIN_8
#define DS_1W_3_GPIO_Port GPIOC
#define DS_1W_4_Pin GPIO_PIN_9
#define DS_1W_4_GPIO_Port GPIOC
#define SD_nEN_PWR_Pin GPIO_PIN_8
#define SD_nEN_PWR_GPIO_Port GPIOA
#define DEBUG_TX_Pin GPIO_PIN_9
#define DEBUG_TX_GPIO_Port GPIOA
#define DEBUG_RX_Pin GPIO_PIN_10
#define DEBUG_RX_GPIO_Port GPIOA
#define FT800_nCS_Pin GPIO_PIN_11
#define FT800_nCS_GPIO_Port GPIOA
#define SD_SPI_nEN_Pin GPIO_PIN_12
#define SD_SPI_nEN_GPIO_Port GPIOA
#define JTMS_SWDIO_Pin GPIO_PIN_13
#define JTMS_SWDIO_GPIO_Port GPIOA
#define JTCK_SWCLK_Pin GPIO_PIN_14
#define JTCK_SWCLK_GPIO_Port GPIOA
#define JTDI_Pin GPIO_PIN_15
#define JTDI_GPIO_Port GPIOA
#define FT800_nEN_PWR_Pin GPIO_PIN_10
#define FT800_nEN_PWR_GPIO_Port GPIOC
#define FT800_nPD_Pin GPIO_PIN_12
#define FT800_nPD_GPIO_Port GPIOC
#define BACKLIGHT_nEN_PWR_Pin GPIO_PIN_2
#define BACKLIGHT_nEN_PWR_GPIO_Port GPIOD
#define JTDO_Pin GPIO_PIN_3
#define JTDO_GPIO_Port GPIOB
#define NJTRST_Pin GPIO_PIN_4
#define NJTRST_GPIO_Port GPIOB
#define SD_nUNLOCK_Pin GPIO_PIN_5
#define SD_nUNLOCK_GPIO_Port GPIOB
#define OUT1_Pin GPIO_PIN_6
#define OUT1_GPIO_Port GPIOB
#define OUT2_Pin GPIO_PIN_7
#define OUT2_GPIO_Port GPIOB
#define EN_3_3VD_Pin GPIO_PIN_8
#define EN_3_3VD_GPIO_Port GPIOB
#define SD_nCS_Pin GPIO_PIN_9
#define SD_nCS_GPIO_Port GPIOB
/* USER CODE BEGIN Private defines */

#define SPI1_MISO_GPIO_Port GPIOA
#define SPI1_MISO_Pin GPIO_PIN_6
#define SPI1_MOSI_GPIO_Port GPIOA
#define SPI1_MOSI_Pin GPIO_PIN_7
#define SPI1_SCK_GPIO_Port GPIOA
#define SPI1_SCK_Pin GPIO_PIN_5


/* USER CODE END Private defines */

#ifdef __cplusplus
}
#endif

#endif /* __MAIN_H */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
