#include "sim800.h"
#include "main.h"
#include <string.h>

char Buf[100];

#define SIM800_LINE_ECHO 1
#define SIM800_LF_CRLF 1

/*----------------------------------------------*/
/* Get a line from the input                    */
/*----------------------------------------------*/

int simgets (		/* 0:End of stream, 1:A line arrived */
	char* buff,	/* Pointer to the buffer */
	int len,		/* Buffer length */
	int Timeout
)
{
	static	uint32_t tickstart = 0U;
	static	int i;
	static	char c;

	/* Init tickstart for timeout managment */
  tickstart = HAL_GetTick();
	
	i = 0;
	
	while (1) 
	{
		while(1)
		{
			if(bSIM800Buffer)
			{
				bSIM800Buffer = 0;

				c = *pSIM800LastCommand;
				if( pSIM800LastCommand++ >= (SIM800Buffer + sizeof(SIM800Buffer) - 1))
					pSIM800LastCommand = SIM800Buffer;

				if(pSIM800LastCommand != pSIM800Buffer)
					bSIM800Buffer = 1;
				
				break;
			}
			
			if (  (HAL_GetTick() - tickstart ) > Timeout )
			{
				return -1;
			}
			
			CriticalTasks();
			
		}
		
		if (!c) return 0;			/* End of stream? */

		if (c == '\r' && i == 0 || c =='\n' && i == 0)	//Leere Zeile
			continue;

		if (c == '\r' || c == '>') break;		/* End of line? or Input Request*/
		
		if (c == '\b' && i) {		/* Back space? */
			i--;
#if SIM800_LINE_ECHO
			HAL_UART_Transmit(&huart1, (uint8_t*) &c, 1,1000);
#endif
			continue;
		}
		if (c >= ' ' && i < len - 1) {	/* Visible chars */
			buff[i++] = c;
#if SIM800_LINE_ECHO
			HAL_UART_Transmit(&huart1, (uint8_t*) &c, 1,1000);
#endif
		}
	}
	buff[i] = 0;	/* Terminate with a \0 */
#if SIM800_LINE_ECHO
#if !SIM800_LF_CRLF
		printf("\r");
#endif
		printf("\n");
#endif
	return 1;
}


int sim800on(void)
{
	
	SIM800_PWRKEY_LOW();
	HAL_Delay(1100);
	SIM800_PWRKEY_HIGH();
	
	simgets(Buf,100,10000);
	if(strcmp(Buf,"RDY")!=0)
		return -1;

	simgets(Buf,100,5000);
	if(strcmp(Buf,"+CFUN: 1")!=0)
		return -1;

	simgets(Buf,100,5000);
	if(strcmp(Buf,"+CPIN: READY")!=0)
		return -1;

	simgets(Buf,100,5000);
	if(strcmp(Buf,"Call Ready")!=0)
		return -1;

	simgets(Buf,100,5000);
	if(strcmp(Buf,"SMS Ready")!=0)
		return -1;
	
	return 0;
}

int sim800off(void)
{
	SIM800_PWRKEY_LOW();
	HAL_Delay(1700);
	SIM800_PWRKEY_HIGH();

	
	SIM800_PWRKEY_LOW();
	HAL_Delay(1100);
	SIM800_PWRKEY_HIGH();
	
	simgets(Buf,100,5000);
	if(strcmp(Buf,"NORMAL POWER DOWN")!=0)
		return -1;

	return 0;
}

int SIM800_Transmit(char *szBuf)
{
	int Length;
	
	Length = strlen(szBuf);
	HAL_UART_Transmit(&huart1, (uint8_t *) szBuf, Length, 1000);
	HAL_UART_Transmit(&huart2, (uint8_t *) szBuf, Length, 1000);
	
	return Length;
}

int SendSMS(char* szText)
{
	char szBuf[100];
	
	SIM800Buffer_Flush();
	
	SIM800_Transmit("AT\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;	
	
	SIM800_Transmit("AT+CMGF=1\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;	
	
	SIM800_Transmit("AT+CSCS=\"GSM\"\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;

	snprintf(szBuf,100, "AT+CMGS=\"%s\"\r\n", Settings.SMSNumber);
	SIM800_Transmit(szBuf);
	simgets(Buf,100,2000);
	
	SIM800_Transmit(szText);
	simgets(Buf,100,2000);
	char lt[2];
	lt[0]=26;
	lt[1]=0;
	SIM800_Transmit(lt);
	SIM800_Transmit("\r\n");
	simgets(Buf,100,2000);

	return 0;
	
}

int SendEmail(char* szText)
{
	SIM800Buffer_Flush();

	SIM800_Transmit("AT\r\n");
	simgets(Buf,100,1000);
	
	if(strcmp(Buf,"OK")!=0)
	{
		if(sim800on())
		{
			return -1;
		}
		else
		{
			HAL_Delay(5000);
		}
	}

	SIM800_Transmit("AT+SAPBR=3,1,\"APN\",\"CMNET\"\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;

	SIM800_Transmit("AT+SAPBR=1,1\r\n");
	simgets(Buf,100,5000);
//	if(strcmp(Buf,"OK")!=0)
//		return -1;

	SIM800_Transmit("AT+EMAILCID=1\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;

	SIM800_Transmit("AT+EMAILTO=30\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;
	
	SIM800_Transmit("AT+EMAILSSL=2\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;

	//SIM800_Transmit("AT+SMTPSRV=\"smtp.gmail.com\"\r\n");
	snprintf(Buf, 100, "AT+SMTPSRV=\"%s\"\r\n",Settings.SMTPServer);
	SIM800_Transmit(Buf);
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;	
	
	//SIM800_Transmit("AT+SMTPAUTH=1,\"ulrich.schratter\",\"vmrqe1ss\"\r\n");
	snprintf(Buf, 100, "AT+SMTPAUTH=1,\"%s\",\"%s\"\r\n",Settings.SMTPUser, Settings.SMTPPass);
	SIM800_Transmit(Buf);
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;	

//	SIM800_Transmit("AT+SMTPFROM=\"ulrich.schratter@gmail.com\",\"Ulrich Schratter\"\r\n");
	snprintf(Buf, 100, "AT+SMTPFROM=\"%s\",\"%s\"\r\n",Settings.EmailFrom, Settings.FromName);
	SIM800_Transmit(Buf);
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;		

//	SIM800_Transmit("AT+SMTPRCPT=0,0, \"schratter.spenger@gmail.com\",\"Ulrich\"\r\n");
	snprintf(Buf, 100, "AT+SMTPRCPT=0,0, \"%s\",\"%s\"\r\n",Settings.EmailTo, Settings.ToName);
	SIM800_Transmit(Buf);
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;
	
	snprintf(Buf, 100, "AT+SMTPSUB=\"%s\"\r\n","Stockwaage");
	SIM800_Transmit(Buf);
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;

	SIM800_Transmit("AT+SMTPBODY=23\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"DOWNLOAD")!=0)
		return -1;	

	SIM800_Transmit("Email from Stockwaage\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;	

	SIM800_Transmit("AT+SMTPSEND\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;
	simgets(Buf,100,20000);
	if(strcmp(Buf,"+SMTPSEND: 1")!=0)
		return -1;

	return 0;
}

int sendthingspeak(void)
{
	char szBuf[200];

	SIM800Buffer_Flush();
	
	SIM800_Transmit("AT\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;

	SIM800_Transmit("AT+CPIN?\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"+CPIN: READY")!=0)
		return -1;
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;
	
	SIM800_Transmit("AT+CIPSHUT\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"SHUT OK")!=0)
	return -1;	
	
	SIM800_Transmit("AT+CREG?\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"+CREG: 0,5")!=0)
		return -1;
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;

	SIM800_Transmit("AT+CGATT?\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"+CGATT: 1")!=0)
		return -1;
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;
	
	SIM800_Transmit("AT+CIPSTATUS\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;
	simgets(Buf,100,5000);
	if(strcmp(Buf,"STATE: IP INITIAL")!=0)
		return -1;	

	SIM800_Transmit("AT+CIPMUX=0\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;	

	SIM800_Transmit("AT+CSTT=\"webaut\",\"\",\"\"\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;	

	SIM800_Transmit("AT+CIICR\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"OK")!=0)
		return -1;
	
	SIM800_Transmit("AT+CIFSR\r\n");
	simgets(Buf,100,5000);

	SIM800_Transmit("AT+CIPSTART=\"TCP\",\"api.thingspeak.com\",\"80\"\r\n");
	simgets(Buf,100,5000);
	simgets(Buf,100,5000);
	
	SIM800_Transmit("AT+CIPSEND\r\n");
	simgets(Buf,100,2000);
	
//	snprintf(szBuf, 100, "GET https://api.thingspeak.com/update?api_key=47JYET9C1BW7V8V7&field%1u=%f&field%1u=%f\r\n",1,Weight,2,Temp);
	snprintf(szBuf, 150, "GET https://api.thingspeak.com/update?api_key=%s&field%1u=%f&field%1u=%f&field%1u=%f&field%1u=%f\r\n",Settings.ThingSpeakKey,1,fweightcalibrated,2,ftemp1,3,DHTHumidity,4,fBatVoltage);
	SIM800_Transmit(szBuf);

	simgets(Buf,100,2000);
	char lt[2];
	lt[0]=26;
	lt[1]=0;
	SIM800_Transmit(lt);
	SIM800_Transmit("\r\n");
	simgets(Buf,100,2000);
	simgets(Buf,100,2000);
	simgets(Buf,100,2000);

	SIM800_Transmit("AT+CIPSHUT\r\n");
	simgets(Buf,100,5000);
	if(strcmp(Buf,"SHUT OK")!=0)
		return -1;	
	
	
	return 0;
}

